<?php 
/**
 * Anchor block.
 *
 * @package     concrete5 v5.7.5+
 * @version     1.0.1
 * @license     MIT License
 * @author      SYSTAKE - Takahiko Takei
 * @copyright   (c) 2015 SYSTAKE.
 * @link        https://systake.com/
 */

namespace Concrete\Package\SimpleAnchor\Block\Anchor;

use Concrete\Core\Block\BlockController;
use Core;
use Page;
use Less_Parser;
use Less_Tree_Rule;

defined('C5_EXECUTE') or die(_("Access Denied."));

/**
 * Block controller class.
 *
 * @author      SYSTAKE - Takahiko Takei
 * @copyright   (c) 2015 SYSTAKE.
 * @link        https://systake.com/
 * @extends     Concrete\Core\Block\BlockController
 */
class Controller extends BlockController
{
    /**
     * The block's primary database table.
     * ブロックで使用する主なデータベーステーブル名
     *
     * @var string
     */
    protected $btTable = 'btContentAnchor';

    /**
     * The block type set handle.
     * ブロックタイプセットハンドル
     *
     * @var string 'basic|navigation|form|social|multimedia'
     */
    protected $btDefaultSet = 'navigation';

    /**
     * The width in pixels of the modal that holds this block when it is added or edited.
     * ブロックの追加/編集する際のモーダルの横幅
     *
     * @var integer
     */
    protected $btInterfaceWidth = 400;

    /**
     * The width in pixels of the modal that holds this block when it is added or edited.
     * ブロックの追加/編集する際のモーダルの高さ
     *
     * @var integer
     */
    protected $btInterfaceHeight = 500;

    /**
     * The block's database record data will be cached.
     * データベースに保存されたブロックのデータをキャッシュ
     *
     * @var boolean
     */
    protected $btCacheBlockRecord = true;

    /**
     * The output of the block will be cached.
     * ブロックの表示結果をキャッシュ
     *
     * @var boolean
     */
    protected $btCacheBlockOutput = true;

    /**
     * The value in seconds that the cache will last before being refreshed.
     * キャッシュがクリアされるまでの秒数
     *
     * @var integer
     */
    protected $btCacheBlockOutputLifetime = 0;

    /**
     * This determines whether a block will cache its output on POST.
     * POSTリクエスト表示の場合にもブロックの表示結果をキャッシュ
     *
     * @var boolean
     */
    protected $btCacheBlockOutputOnPost = true;

    /**
     * This determines whether a block will cache its output if the current user is logged in.
     * ログインユーザー表示の場合にもブロックの表示結果をキャッシュ
     *
     * @var boolean
     */
    protected $btCacheBlockOutputForRegisteredUsers = false;

    /**
     * When this block is exported, any columns will automatically be swapped for links to specific pages.
     * ブロック出力時に自動的に特定ページへのリンクへ差し替えるフィールド指定
     *
     * @var array
     */
    protected $btExportPageColumns = array('internalLinkCID');

    /**
     * To load the helpers.
     * 読み込むヘルパー
     *
     * @var array
     */
    public $helpers = array('form');

    /**
     * Returns the translated block description.
     * 翻訳されたブロックの概要を取得
     *
     * @return string
     */
    public function getBlockTypeDescription()
    {
        return t("Creates the anchor tag from a page on your site or specify the URL.");
    }

    /**
     * Returns the translated name of the block.
     * 翻訳されたブロックの名称を取得
     *
     * @return string
     */
    public function getBlockTypeName()
    {
        return t("Anchor");
    }

    /**
     * The block's template is rendered.
     * ブロックの表示
     */
    public function view()
    {
        $this->set('linkURL', $this->getLinkURL());
        $this->set('target', ($this->targetBlank) ? '_blank' : '_self');
    }

    /**
     * The add template on block is rendered.
     * ブロックの追加画面の表示
     */
    public function add()
    {
        $this->edit();
    }

    /**
     * The edit template on block is rendered.
     * ブロックの編集画面の表示
     */
    public function edit()
    {
        $this->requireAsset('css', 'font-awesome');
        $classes = $this->getIconClasses();

        // let's clean them up
        $icons = array('' => t('Choose Icon'));
        $txt   = Core::make('helper/text');
        foreach ($classes as $class) {
            $icons[$class] = $txt->unhandle($class);
        }
        $this->set('icons', $icons);
    }

    /**
     * This validate is automatically run when a block is submitted to the backend to be saved.
     * ブロックの保存時に自動的にバリデーションを実行
     *
     * @param array $args
     */
    public function validate($args)
    {
        $e = Core::make('helper/validation/error');

        if (mb_strlen($args['label'], APP_CHARSET) > 255) {
            $e->add(t('Labels are limited to a maximum of 255 characters.'));
        }

        switch (isset($args['linkType']) ? intval($args['linkType']) : 0) {
            case 1:
                if ($args['internalLinkCID'] == 0) {
                    $e->add(t('You must choose a page.'));
                }
                break;
            case 2:
                if (empty($args['externalLink'])) {
                    $e->add(t('You must enter a URL.'));
                }
                if (mb_strlen($args['externalLink'], APP_CHARSET) > 255) {
                    $e->add(t('URLs are limited to a maximum of 255 characters.'));
                }
                if (!preg_match('/^(https?)+(:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+)$/', $args['externalLink'])) {
                    $e->add(t('You must enter the correct URL.'));
                }
                break;
            default:
                $e->add(t('You need to set the link.'));
                break;
        }

        if ($e->has()) {
            return $e;
        }
    }

    /**
     * This method is automatically run when a block is submitted to the backend to be saved.
     * ブロックの保存時に自動的に実行
     *
     * @param array $args
     */
    public function save($args)
    {
        switch (isset($args['linkType']) ? intval($args['linkType']) : 0) {
            case 1:
                $args['externalLink']    = '';
                break;
            case 2:
                $args['internalLinkCID'] = 0;
                break;
            default:
                $args['externalLink']    = '';
                $args['internalLinkCID'] = 0;
                break;
        }
        unset($args['linkType']);
        $args['targetBlank'] = $args['targetBlank'] ? 1 : 0;
        parent::save($args);
    }

    /**
     * Retrieve link URL.
     * リンクURLの取得
     *
     * @return string
     */
    public function getLinkURL()
    {
        if (!empty($this->externalLink)) {
            return $this->externalLink;
        } else {
            if (!empty($this->internalLinkCID)) {
                $linkToC = Page::getByID($this->internalLinkCID);
                return (empty($linkToC) || $linkToC->error) ? '' : Core::make('helper/navigation')->getLinkToCollection($linkToC);
            } else {
                return '';
            }
        }
    }

    /**
     * Retrieve class name of icons from less file FontAwesome.
     * FontAwesomeのLessファイルからアイコンのclass名を取得
     *
     * @return array
     */
    protected function getIconClasses()
    {
        $iconLessFile = DIR_BASE_CORE . '/css/build/vendor/font-awesome/variables.less';
        $icons        = array();

        $l      = new Less_Parser();
        $parser = $l->parseFile($iconLessFile, false, true);
        $rules  = $parser->rules;

        foreach ($rules as $rule) {
            if ($rule instanceof Less_Tree_Rule) {
                if (strpos($rule->name, '@fa-var') === 0) {
                    $name    = str_replace('@fa-var-', '', $rule->name);
                    $icons[] = $name;
                }
            }
        }
        asort($icons);

        return $icons;
    }
}
